/*****************************************************************************/
/*                                                                           */
/*   Demo:                                                                   */
/*   LCD Back Light Control with AVR                                         */
/*                                                                           */
/*   (c) 2008 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik for Elektor   */
/*   http://www.reusch-elektronik.de                                         */
/*   http://reweb.fh-weingarten.de/elektor                                   */
/*                                                                           */
/*   Atmel AVR Studio V4.13.528                                              */
/*   GNU C-Compiler GCC V4.2.20                                              */
/*                                                                           */
/*   Created: 2008-02-19                                                     */
/*   Changed: 2008-02-21                                                     */
/*                                                                           */
/*****************************************************************************/

// Preferences:
// CPU: ATmega32
// Clock frequency: 8.0MHz 
// PWM output (backlight current): OC2 (PD7)
// Analogue input (current feedback): ADC0 (PA0)

// For demonstration a button has to be connected to PB4.
// Backlight will increase, while button is pressed.
// LED on PB0 will flash with 0,5Hz as control.

#define feedback				// comment this line, if you want to test pwm mode only

#include <avr/interrupt.h>
#include "backlight.h"

// Port assignments for demo
#define DemoDir		DDRB	// data direction register of port used for demo
#define OutPort		PORTB	// output
#define InPort		PINB	// input
#define BUT				PB4		// input button switch
#define LED				PB0		// output LED

// Timer 1 (used in demo)
#define IRQS_PER_SECOND   1000									// generated timer 1 interrupts per second
#define TIME20MS					IRQS_PER_SECOND / 50	// counter value for 20ms
uint16_t TimerCount;														// counter for timer1 interrupts
uint16_t Timer20ms;                             // counter for 20ms timer

//-----------------------------------------------------------------------------
void Init_Timer1(void)
//-----------------------------------------------------------------------------
// Init timer1 for IRQS_PER_SECOND 
{
  TCCR1A=0;	// Timer1: no PWM
  // Timer1 is counter: Clear Timer on Compare Match (CTC, Mode #4)
  // Timer1 is running with MCU clock / 64 (prescale=64)
	TCCR1B=(1<<WGM12)|(1<<CS11)|(1<<CS10);
  // OutputCompare for Timer1 frequency
  OCR1A = (unsigned short) ((unsigned long) F_CPU / 64 / IRQS_PER_SECOND -1);
  // activate OutputCompareA-Interrupt for Timer1
  TIMSK|=(1<<OCIE1A);
}

//-----------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect)
//-----------------------------------------------------------------------------
{	// ISR for Timer1
	// 20ms timer
	Timer20ms++;
	if (Timer20ms>=TIME20MS)
	{
		Timer20ms=0;
		if (!(InPort&(1<<BUT)))
		{	// button is pressed
			#ifdef feedback
			Current++;  // increment backlight current
			#else
			SetPWM(PWMValue+1);  // increment pulse width
			#endif
		}
	}
	// second timer
	TimerCount++;
	if (TimerCount>=IRQS_PER_SECOND)
	{	// 1 second passed
		TimerCount=0;
		OutPort^=(1<<LED);	// toggle LED
	}
}

//-----------------------------------------------------------------------------
int main(void)
//-----------------------------------------------------------------------------
{
	#ifdef feedback
	Init_BacklightFB(0);	// init backlight control with off state (feedback mode)
	#else
	Init_Backlight(0);		// init backlight control with off state (pwm mode)
	#endif

	// Demo
	TimerCount=0;			// reset timer1 interrupt counter
	Timer20ms=0;      // reset 10ms timer
	DemoDir|=1<<LED;	// set LED port as output
	Init_Timer1();		// init timer1, used for demo

	sei();	// enable interrupts

	for (;;);	// never ending loop
}
